#ifndef S9X_CONTROLLER_H
#define S9X_CONTROLLER_H

#include "defines.h"
#include "frend.h"
#include "misc.h"
#include "skins.h"
#include <string>

#ifndef S9X_JS_COUNT
#define S9X_JS_COUNT 2
#endif

namespace s9x {

class JoyMap;
enum
{
  A_BTN, B_BTN, X_BTN, Y_BTN,
  L_BTN, R_BTN, START_BTN, SELECT_BTN,
  GAMEPAD, BUTTONPAD, PADTEXT, PADCOLORCOUNT
};
typedef frend::Colors PadColors[PADCOLORCOUNT];

class Controller : public Notepage
{
 private:
	frend::Window       _dev_window, _map_window;
	frend::File         _dev_js0, _dev_js1, _dev_js2, _dev_js3, *_dev_js[S9X_JS_COUNT];
	frend::Button       _dev_btn, _dev_close_btn, _map_btn, _map_close_btn;
	frend::PulldownMenu _buttons;
	frend::Checkbox     _old, _use_joystick, _swap;
	frend::Element*     _map_diagram;
	frend::Box          _detail_box;
	frend::ButtonBox    _map_buttonbox;
	PadColors           _default_padcolors, _padcolors;
	JoyMap              *_map[S9X_JS_COUNT];
	void eventOccurred(frend::Event*e);
 public:
	Controller(frend::Notebook*parent);
	virtual ~Controller();
	void setToDefaults();
	void set9xVersion(float version);
	void siftArgs(frend::ArgList&);
	void compileArgs(frend::ArgList&);
	void applySkin(Skin*);
	const PadColors& getColors() const { return _padcolors; }
};

class JoyMap : 	public frend::Option, private frend::Listener
{
 private:
	int             _index, _row, _selected_btn, _axes, _buttons;
	int             _buttonmap[GAMEPAD];
	int             _device_fd, _device_id;
	bool            _is_visible;
	std::string     _js_dev, _js_name, _js_err;
	frend::TextArea _details;
	const PadColors &_clr;
	void _setBtn(int btn, int val);
	int  _getBtn(int btn);
	void eventOccurred(frend::Event*e);
 public:
	JoyMap(Controller*parent, int j, frend::Box&);
	void setToDefault();
	bool isDefault();
	void siftArgs(frend::ArgList& L);
	void compileArgs(frend::ArgList& L);
	void setSelectedBtnVal(int val);
	int  getSelectedBtnVal();
	void updateDetails();
	void moveSelector(int dir);
	int  openDevice(const std::string& dev);
	void closeDevice();
};

}

#endif
