/*
 * snes9express
 * cpu.cc
 * Copyright  1998-2009  David Nordlund
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * For further details, please read the included COPYING file,
 * or go to http://www.gnu.org/copyleft/gpl.html
 */

#include "cpu.h"

namespace fr = frend;

namespace s9x {

CPU::CPU(fr::Notebook*parent):
Notepage(parent, "CPU"),
_hdma(this, "H-DMA", true),
_speedhacks(this, "Speed-Hacks", true),
_superfx(this, "SuperFX", true),
_dsp1(this, "DSP1", true),
_cycles(this, "Cycles:", fr::HORIZONTAL, 100, 0, 200, 0, true)
{
	setGridSize(4, 2, false);

	_hdma.args << fr::CASE_INSENSITIVE << "-hdma" << "-ha";
	_hdma.not_args << fr::CASE_INSENSITIVE << "-nohdma" << "-nh";
	_hdma.setKeystroke("0");
	_hdma.setTooltip("Use H-DMA emulation");
	pack(_hdma);
	addOption(_hdma);

	_speedhacks.args << fr::CASE_INSENSITIVE << "-speedhacks" << "-SH";
	_speedhacks.not_args << fr::CASE_INSENSITIVE << "-nospeedhacks";
	_speedhacks.setTooltip("Use some speed-up shortcuts.  These can "
	    "occasionally cause problems with certain ROMs");
	pack(_speedhacks);
	addOption(_speedhacks);

	_superfx.args << fr::CASE_INSENSITIVE << "-sfx" << "-superfx";
	_superfx.not_args << fr::CASE_INSENSITIVE << "-nosfx" << "-nosuperfx";
	_superfx.setTooltip("Allow emulation of the SuperFX chip");
	pack(_superfx);
	addOption(_superfx);

	_dsp1.args << fr::CASE_INSENSITIVE << "-dsp1";
	_dsp1.not_args << fr::CASE_INSENSITIVE << "-nodsp1";
	_dsp1.setTooltip("Allow emulation of the DSP1 chip");
	pack(_dsp1);
	addOption(_dsp1);

	_cycles.args << fr::CASE_INSENSITIVE << "-cycles" << "-h";
	_cycles.setTooltip("The percent of CPU cycles to execute per scan line.  Lower is faster, but too low may cause some ROMs not to work");
	pack(_cycles, 0, 1, 2, 2);
	addOption(_cycles);
}

void CPU::set9xVersion(float version)
{
	_dsp1.enableIf(version >= 1.05);
}

void CPU::eventOccurred(fr::Event*e)
{
}

}
