/*
 * snes9express
 * extra.cc
 * Copyright  1998-2009  David Nordlund
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * For further details, please read the included COPYING file,
 * or go to http://www.gnu.org/copyleft/gpl.html
 */

#include <cstdlib>
#include <cstdio>
#include <string>
#include <time.h>
#include "extra.h"

namespace fr = frend;

namespace s9x {

Snapshot::Snapshot(fr::Notebook*parent):
Notepage(parent, "Snapshot"),
_desc(this, "If you took a snaphot during a game,\n"
            "you can resume playing from that spot"),
_file(this, _name, "", false)
{
	setGridSize(1, 2, true);

	pack(_desc);

	_file.addLabel("Snapshot file:");
	_file.args << fr::CASE_INSENSITIVE << "-loadsnapshot";
	_file.setKeystroke("Load: [F1-F10] / Save: Shift+[F1-F10]");
	_file.setTooltip("if you have a snes9x snapshot that you'd like to "
	                 "load, specify it here.");
	_file.addListener(this);
	pack(_file);
	addOption(_file);

	addListener(this);
}

void Snapshot::eventOccurred(fr::Event*e)
{
	if( e->is(_file, fr::CLICK) || e->is(this, fr::MENU_CLICK) )
	{
		if( _file.getFilename().size() == 0 )
		{
			const char*snap_env = getenv(S9X_SNAPSHOTENV);
			if( snap_env )
				_file.setDefaultPath( snap_env );
		}
		//std::cerr << _file.getName() << "=\"" << _file.getFilename() << "\" [" << _file.getDefaultPath() << ']' << std::endl;
		
		if(e->is(this, fr::MENU_CLICK))
			_file.filePopup();
	}
}

/* ############################# s9x_Extra ################################ */

Extra::Extra(fr::Notebook*parent):
Notepage(parent, "Extra"),
_desc(this, "Any snes9x options you want to use that\n"
            "aren't present in " PROG ", you can add here."),
_extra_args(this, "Extra Options:", "", false)
{
	setGridSize(1, 2, true);
	pack(_desc);
	_extra_args.setTooltip("Add here any options not covered elsewhere in " PROG);
   pack(_extra_args);
}

void Extra::setToDefaults()
{
	_argstr = "";
	_extra_args.setText(_argstr);
}

void Extra::siftArgs(fr::ArgList& l)
{
	_argstr = "";
	int p = l.countArgs();
	for(int a=0; a < p; a++)
	{
		if(!l.isMarked(a))
		{
			const char *s = l[a];
			if( s && s[0] )
			{
				_argstr += s;
				_argstr += ' ';
			}
		}
	}
	_extra_args.setText(_argstr);
}

void Extra::compileArgs(fr::ArgList& l)
{
	if(!_extra_args.isDefault())
	{
		fr::ArgList extras( _extra_args.getText().c_str() );
		for(int i=0, c=extras.countArgs(); i < c; i++)
			l << extras[i];
	}
}

void Extra::eventOccurred(fr::Event*e)
{
}

void randomTip()
{
#define EXCUSE(n, s) "Excuse for excessive game playing, #" n ":\n \"" s "\""
	static const char*TIPS[] =
	{
		"If you are having trouble with a ROM that uses SuperFX,\n"
			"try setting the ROM Format to \"Interleaved 2\".",

		"If you would like to know the exact command snes9express\n"
			"is sending to snes9x, open the Profiler and pull the\n"
			"scroll-bar over to the right",

		"Using the Control or Shift keys, you can select multple\n"
			"profiles in the Profiler.  Pressing Apply will merge the\n"
			"the selected profiles.",

		"If you have a OpenGL supported video card,\n"
			"try using the OpenGL version of snes9x,\n"
			"by selecting it in the Preferences window.",

		"If the joystick button mapper can't find a joystick,\n"
			"try clicking the Devices button and change the device file\n"
			"from /dev/jsN to /dev/input/jsN (where N is the joystick number)",

		"If, for some bizarre reason, you want to play with a really old\n"
			"version of snes9x, you can set the snes9x version number in the\n"
			"Preferences window.  The " PROG " interface will show the options\n"
			"that were available in that version of snes9x.",

		EXCUSE("1", "I am trying to improve my hand-eye coordination."),
		EXCUSE("2", "This game sure helps my situation analysis skills."),
		EXCUSE("3", "Video games are a great way to exercise sharp reflexes,\n"
		    "and shorten reaction and decision making times"),
		EXCUSE("4", "Game? No, I think this is some kind of\n"
		    "computer virus that has taken over the screen.\n"
		    "I've been trying to get rid of it all day."),
		EXCUSE("5", "Admit it.  You are just jealous that I am so much\n"
		    "better at this than you.  It's driving you crazy."),
		EXCUSE("6", "Many thousands of man-hours went into the creation\n"
		    "of this game.  The least I can do is appreciate all that\n"
		    "hard work, even if for just a fraction of that amount of time.")

			"If you can think of any tips that should go here,\n"
			"please feel free to submit them to " SNES9EXPRESS_EMAIL,
		""
	};
#undef EXCUSE
	int tipcount=-1;
	while( TIPS[++tipcount][0] );	// count # of tips
	srand(time(NULL));
	int t = rand() % tipcount;
	fr::mesg(TIPS[t]);
}

}
