#ifndef S9X_INTERFACE_H
#define S9X_INTERFACE_H

namespace s9x
{
	class Interface;
}

#include <string>
#include "defines.h"
#include "frend.h"
#include "skins.h"
#include "rom.h"
#include "sound.h"
#include "video.h"
#include "controller.h"
#include "cpu.h"
#include "netplay.h"
#include "extra.h"
#include "profiler.h"
#include "prefs.h"
#include "misc.h"

namespace s9x
{

class Interface: public frend::MainProgram, private frend::Listener
{
 friend class Prefs;
 private:
	frend::Notebook		_notebook;
	frend::Element		_quit, _tip;
	frend::CustomButton	_power_btn, _reset_btn, _eject_btn;
	frend::ButtonBox	_console_buttons;
	frend::Button		_power_btn2, _reset_btn2, _eject_btn2; // skinless version
	frend::Size			_default_notebook_size, _default_button_size, _default_window_size;
	frend::Point		_console_button_place;
	frend::ArgList		_last_args;
	File				_log_file;
	frend::Window		_about_window, _log_window;
	frend::TextArea		_about_text, _log_text;
	frend::Button		_about_close_btn, _log_close_btn;
	frend::Label		_log_label;
	Skin                *_skin;
	std::string			_outbuf, _errbuf;
	void _createMenu();
	void _createNotebook();
	void _createAboutInfo();
	void eventOccurred(frend::Event*e);
 public:
	Prefs		prefs;
	ROM			rom;
	Sound		sound;
	Video		video;
	CPU			cpu;
	Controller	controller;
	NetPlay		netplay;
	Snapshot	snapshot;
	Extra		extra;
	Profiler	profiler;

	Interface(const frend::ArgList& args);
	~Interface();
	void hitPower();
	void hitEject();
	void hitReset();
	void viewOutput(const std::string& msg="");
	void play();
	void setToDefaults();
	void set9xVersion(double version);
	void siftArgs(frend::ArgList&);
	void compileArgs(frend::ArgList&);
	void setROMdir(const std::string& d) { rom.setROMdir(d); }
	void applySkin(const std::string& skindir);
	void shedSkin();
};

}

#endif
