#ifndef S9X_MISC_H
#define S9X_MISC_H

#include <string>
#include <fstream>
#include <list>
#include <sys/stat.h>
#include "frend.h"
#include "skins.h"

namespace s9x
{

class OptionManager
{
 private:
  std::list<frend::Option*> _options;

 public:
  OptionManager();
  virtual ~OptionManager();
  virtual void addOption(frend::Option& o);
  virtual void setToDefaults();
  virtual void siftArgs(frend::ArgList& L);
  virtual void compileArgs(frend::ArgList& L);
};

class Notepage: public frend::Notepage, public OptionManager, protected frend::Listener
{
 protected:
  virtual void eventOccurred(frend::Event*e) = 0;
  virtual void _pageChanged(bool to_me);
  virtual std::string _getTabIconID() const;
 public:
  Notepage(frend::Notebook*parent, const char*name);
  virtual void applySkin(Skin*S);
};


class File
{
  private:
    std::string _filepath;
    const char* _filepath_cstr;
    struct stat _statbuf;
    bool _existant;
    
  public:
    File(const std::string& filename);
    virtual ~File();

    std::string getFilepath() const     { return _filepath; }
    bool exists() const { return _existant; }
    bool isFile() const { return (_statbuf.st_mode & S_IFREG) != 0; }
    bool isDir() const  { return (_statbuf.st_mode & S_IFDIR) != 0; }
    void open(std::ifstream& i);
    void open(std::ofstream& o);
    void close(std::ifstream& f);
    void close(std::ofstream& f);
    void remove();
    int  stat(struct stat *buf=0);
    friend std::string operator+(const std::string& s, const File& f);
    friend std::string operator+(const File& f, const std::string& s);
    friend std::string operator+(const char* s, const File& f);
};

void createS9xDir();	///< create the S9X_SUBDIR if it does not yet exist.
void removeS9xDir();	///< remove the S9X_SUBDIR if it contains no files.

}

#endif
