/*
 * snes9express
 * netplay.cc
 * Copyright  1998-2009  David Nordlund
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * For further details, please read the included COPYING file,
 * or go to http://www.gnu.org/copyleft/gpl.html
 */

#include <cstdlib>
#include "netplay.h"

namespace fr = frend;

namespace s9x
{

NetPlay::NetPlay(fr::Notebook*parent):
Notepage(parent, "NetPlay"),
_use_netplay(this, "Use NetPlay", false),
_server(this, "Server:", "", false),
_port(this, "Port:", S9X_NETPLAY_PORT, 1024, 65536, 0)
{
	setGridSize(2, 2, false);

	_use_netplay.args << fr::CASE_INSENSITIVE << "-net";
	_use_netplay.setTooltip("Play a multiplayer game on the network");
	_use_netplay.addListener(this);
	pack(_use_netplay, 2, 1);
	addOption(_use_netplay);

	_server.args << fr::CASE_INSENSITIVE << "-server" << "-srv";
	_server.setTooltip("The name of the host running the s9xserver");
	pack(_server);
	addOption(_server);

	_port.args << fr::CASE_INSENSITIVE << "-port" << "-po";
	_port.setTooltip("The port of the host to connect to in a NetPlay game.");
	pack(_port);
	addOption(_port);
}

void NetPlay::setToDefaults()
{
	int port = 0;
	const char *env_server, *env_port;

	_use_netplay.setToDefault();

	env_server = getenv("S9XSERVER");
	if(!env_server)
		env_server = "";
	_server.setText(env_server);

	env_port = getenv("S9XPORT");
	if(env_port)
		port = atoi(env_port);
	if(!port)
		port = S9X_NETPLAY_PORT;
	_port.setValue(port);

	if((!_server.isDefault()) || (!_port.isDefault()))
		_use_netplay.setState(true);
	fr::Event(&_use_netplay, fr::CLICK);
}

void NetPlay::set9xVersion(float version)
{
	bool too_old = version < 1.10;
	bool disabled = (version > 1.42) && (version < 1.44);
	bool cond = (!disabled) && (!too_old);
	_use_netplay.enableIf(cond);
	_server.enableIf(cond);
	_port.enableIf(cond);
	fr::Event(&_use_netplay, fr::CLICK);
}

void NetPlay::siftArgs(fr::ArgList& l)
{
	Notepage::siftArgs(l);
	if((!_server.isDefault()) || (!_port.isDefault()))
		_use_netplay.setState(true);
	fr::Event(&_use_netplay, fr::CLICK);
}

void NetPlay::compileArgs(fr::ArgList& l)
{
	if(!_use_netplay.getState())
		return;
	_server.compileArgs(l);
	_port.compileArgs(l);
}

void NetPlay::eventOccurred(fr::Event*e)
{
   bool s;
   if(e->is(_use_netplay))
   {
      s = _use_netplay.getState();
      _server.setEditable(s);
      _port.setEditable(s);
   }
}

} // namespace s9x
