#ifndef S9X_ROM_H
#define S9X_ROM_H

#include "defines.h"
#include "frend.h"
#include "misc.h"
#include "skins.h"
#include <ctime>
#include <string>
#include <map>

namespace s9x {

class ROMdata
{
 private:
	std::string	  _file_id;
	frend::strmap _data;
 public:
	ROMdata(const std::string& file_id);
	~ROMdata();
	bool	      extractROMdata(const std::string& dir, const std::string& file);
	std::string   getID() const { return _file_id; }
	void          setID(const std::string& i) { _file_id = i; }
	void          setData(const std::string& key, const std::string& val) { _data[key] = val; }
	void          setData(const std::string& key, time_t val);
	std::string   getData(const std::string& key, const std::string& fallback="");
	time_t        getData(const std::string& key, time_t fallback);
	void          setROMtitle(const std::string& t) { setData("title", t); }
	std::string	  getROMtitle() { return getData("title"); }
	std::string   getEncoded() { return "-"; }
	friend std::istream& operator>>(std::istream& i, ROMdata& r);
	friend std::ostream& operator<<(std::ostream& o, const ROMdata& r);
};

class ROMselector: public frend::Window, private frend::Listener
{
 typedef std::map<std::string,ROMdata*> ROMDataMap;
 private:
	int              _file_count;
	frend::Label     _rom_icon;
	ROMDataMap       _romdatamap;
	frend::DataTable _rom_list;
	frend::TextArea  _rom_details;
	frend::Button    _ok_btn, _cancel_btn;
	frend::ButtonBox _btnbox;
	File             _romdatafile;
	std::string      _rom_dir, _nowplaying;
	time_t           _play_time;
	bool             _save_needed;
	void        _loadROMdata();
	void        _saveROMdata();
	void        _loadDirectory(const std::string& directory);
	int	        _addROM(const std::string& dir, const std::string& file, bool extractdata);
	ROMdata*    _getROMdata(const std::string& fileid);
	ROMdata*    _getROMdata(int row);
	void        _showROMdetails(int row);
	std::string _timestr(time_t t);
	void		eventOccurred(frend::Event*e);
	bool		_refreshContent();
	void		_deleteContent();
 public:
	ROMselector(frend::Element*parent);
	virtual ~ROMselector();
	std::string getSelectedFilename();
	std::string getSelectedFilepath();
	void        rowSelected(int row);
	void        setVisibility(bool v);
	void        setROMdir(const std::string& d);
	bool        hasROMdata(const std::string& fileid);
	void        doPrePlay(const std::string& f);
	void        doPostPlay();
	static std::string getFileID(const std::string& filename);
	static std::string getTitleFromFilename(const std::string& filename);
};

class ROM: public Notepage
{
 private:
	frend::Button       _rom_btn;
	frend::File         _file;
	frend::PulldownMenu	_memory_map, _format;
	frend::Element*		_bnr;
	ROMselector	        _selector;
	void eventOccurred(frend::Event*e);
 public:
	ROM(frend::Notebook*parent);
	void        set9xVersion(float version);
	void        siftArgs(frend::ArgList&);
	void        setFilename(const std::string&);
	std::string getFilename();
	void        setROMdir(const std::string&);
//	std::string getROMdir() { return _rom_dir; }
	void        filePopup();
	void        applySkin(Skin*);
	void        doPrePlay() { _selector.doPrePlay(getFilename()); }
	void        doPostPlay() { _selector.doPostPlay(); }
};

}

#endif
