#ifndef S9X_S9XSKIN_H
#define S9X_S9XSKIN_H

#include "defines.h"

#include <cstdio>
#include <cstdlib>
#include <stdint.h>
#include <iostream>
#include <string>
#include <map>
#include <list>

namespace s9x
{

#ifdef S9X_SKINABLE

typedef struct
{
	char filename[40];
	uint32_t fileoffset;
	uint32_t filesize;
} SkinIndexEntry;


class SkinFileBase
{
 public:
	SkinFileBase() { _skinfptr = 0; }
	virtual ~SkinFileBase();
	typedef std::map<std::string,SkinIndexEntry*> Index;
 protected:
	Index _idx;
	FILE *_skinfptr;
};

class SkinReader : public SkinFileBase
{
 private:
	void _readHeader();	///< Read file header to verify that the file is a snes9express skin file
	void _loadIndex();	///< Load the index of files contained within the skin file
	SkinIndexEntry *_index_table;
	std::list<std::istream*> _istream_pool;

 public:
	SkinReader(const std::string filespec);	///< Open the skin file for reading.
	virtual ~SkinReader();
	uint32_t getFileSize(std::string filename);	///< @return the number of bytes in the filename requested, 0 if file not found
	std::istream& getIstream(std::string filename);	///< @return an istream for the data in a given filename
     /**
     * Return a FILE pointer starting at the beginning of the requested file.
     * Unfortunately, there's no way to fake an EOF(correct me if I'm wrong, please)
     * so a reader of this can potentially read beyond the file boundry. :/
     */
	FILE* seekFile(std::string filename);
};

class SkinWriter : public SkinFileBase
{
 private:
	typedef std::list<std::string> FileList;
	FileList _files;
	std::string _stylesheet;
	std::string _skindir;

	void _scanStyleSheet();
	void _addFile(const std::string& filename);
	void _writeIndex();
 public:
	SkinWriter(const std::string& inputdir);
	virtual ~SkinWriter();

	void writeFile(const std::string& outputfilename);
};

#else
class SkinReader;
#endif

int skin_main(int argc, char**argv);

} // namespace s9x

#endif
