#ifndef S9X_SKINS_H
#define S9X_SKINS_H

#include <iostream>
#include <map>
#include <string>
#include "frend.h"
#include "s9xskin.h"

namespace s9x
{

#ifndef S9X_INTERFACE_H
class Interface;
#endif

class Skin;

class SkinSelector:	public frend::PulldownMenu
{
 typedef std::map<std::string,std::string> PathMap;
 typedef std::map<frend::MenuItem*,std::string> MenuMap;
 private:
	std::string _defaultskin;
	PathMap     _skinpaths;
	MenuMap     _menuitems;
	void _loadPaths();
	void _fillMenu();
	void _setDefaultSkinFromFile(const std::string& f);
 public:
	SkinSelector(frend::Element*parent);
	virtual ~SkinSelector();
	std::string getSelectedSkinName();
	std::string getSelectedSkinPath();
};

class SkinSection
{
 private:
	Skin* _skin;
	std::map<std::string, std::string> _properties;	// string from source
 public:
	SkinSection(Skin*parent);
	virtual ~SkinSection();
	void          set(const std::string& name, const std::string& value) { _properties[name] = value; }
	bool          has(const std::string& name) const;
	std::string   getString(const std::string name) { return _properties[name]; }
	int           getInt(const std::string& name) const;
	frend::Colors getColors(const std::string& name) const;
	frend::Image* getImage(const std::string& imgname) const;
};

class Skin
{
 private:
	Interface   *_parent;
	std::string _skinfilename;
	SkinReader  *_skinr;
	std::map<std::string, SkinSection*>  _sections;
	std::map<std::string, frend::Image*> _images;	// image cache
	void _load();
 public:
   Skin(Interface*parent, const std::string& skinfile);
   virtual ~Skin();
   std::string     getName() const;
   frend::Element* getElement() { return (frend::Element*)_parent; }
   std::string     getFilename() const { return _skinfilename; }
   bool            equals(const std::string& f) { return _skinfilename==f; }
   SkinSection*    getSection(const std::string& name) { return _sections[name]; }
   frend::Image*   getImage(const std::string& imgname);
};
std::istream& operator>> (std::istream& is, SkinSection* ss);

} // namespace s9x

#endif
